@@ -6,16 +6,20 @@ use ::winapi::{
     um::{
         errhandlingapi::GetLastError,
         handleapi::{CloseHandle, INVALID_HANDLE_VALUE},
-        memoryapi::{MapViewOfFile, UnmapViewOfFile, VirtualQuery, FILE_MAP_READ, FILE_MAP_WRITE},
-        winbase::{CreateFileMappingA, OpenFileMappingA},
+        memoryapi::{
+            CreateFileMappingW, MapViewOfFile, OpenFileMappingW, UnmapViewOfFile, VirtualQuery,
+            FILE_MAP_READ, FILE_MAP_WRITE,
+        },
         winnt::{HANDLE, MEMORY_BASIC_INFORMATION, PAGE_READWRITE},
     },
 };
 
 use crate::ShmemError;
 
-use std::ffi::CString;
+use std::ffi::OsStr;
+use std::iter::once;
 use std::mem::size_of;
+use std::os::windows::ffi::OsStrExt;
 use std::ptr::null_mut;
 
 pub struct MapData {
@@ -51,6 +55,7 @@ impl Drop for MapData {
 
 //Creates a mapping specified by the uid and size
 pub fn create_mapping(unique_id: &str, map_size: usize) -> Result<MapData, ShmemError> {
+    //In addition to being the return value, the Drop impl of this helps clean up on failure
     let mut new_map: MapData = MapData {
         unique_id: String::from(unique_id),
         map_handle: NULL,
@@ -60,16 +65,16 @@ pub fn create_mapping(unique_id: &str, map_size: usize) -> Result<MapData, Shmem
 
     //Create Mapping
     new_map.map_handle = unsafe {
-        let high_size: u32 = (map_size as u64 & 0xFFFF_FFFF_0000_0000 as u64) as u32;
+        let high_size: u32 = ((map_size as u64 & 0xFFFF_FFFF_0000_0000 as u64) >> 32) as u32;
         let low_size: u32 = (map_size as u64 & 0xFFFF_FFFF as u64) as u32;
-        CreateFileMappingA(
+        let unique_id: Vec<u16> = OsStr::new(unique_id).encode_wide().chain(once(0)).collect();
+        CreateFileMappingW(
             INVALID_HANDLE_VALUE,
             null_mut(),
             PAGE_READWRITE,
             high_size,
             low_size,
-            #[allow(clippy::temporary_cstring_as_ptr)]
-            CString::new(unique_id).unwrap().as_ptr(),
+            unique_id.as_ptr(),
         )
     };
     let last_error = unsafe { GetLastError() };
@@ -85,9 +90,6 @@ pub fn create_mapping(unique_id: &str, map_size: usize) -> Result<MapData, Shmem
         unsafe { MapViewOfFile(new_map.map_handle, FILE_MAP_READ | FILE_MAP_WRITE, 0, 0, 0) } as _;
     if new_map.map_ptr.is_null() {
         let last_error = unsafe { GetLastError() };
-        unsafe {
-            CloseHandle(new_map.map_handle);
-        }
         return Err(ShmemError::MapCreateFailed(last_error));
     }
 
@@ -96,6 +98,7 @@ pub fn create_mapping(unique_id: &str, map_size: usize) -> Result<MapData, Shmem
 
 //Opens an existing mapping specified by its uid
 pub fn open_mapping(unique_id: &str) -> Result<MapData, ShmemError> {
+    //In addition to being the return value, the Drop impl of this helps clean up on failure
     let mut new_map: MapData = MapData {
         unique_id: String::from(unique_id),
         map_handle: NULL,
@@ -105,11 +108,11 @@ pub fn open_mapping(unique_id: &str) -> Result<MapData, ShmemError> {
 
     //Open existing mapping
     new_map.map_handle = unsafe {
-        OpenFileMappingA(
+        let unique_id: Vec<u16> = OsStr::new(unique_id).encode_wide().chain(once(0)).collect();
+        OpenFileMappingW(
             FILE_MAP_READ | FILE_MAP_WRITE,
             FALSE as _,
-            #[allow(clippy::temporary_cstring_as_ptr)]
-            CString::new(unique_id).unwrap().as_ptr(),
+            unique_id.as_ptr(),
         )
     };
     if new_map.map_handle as *mut _ == NULL {
